@@ -49,8 +49,7 @@ static Sdb* get_sdb(RBinFile *bf) {
 }
 
 static bool load(RBinFile *bf, RBuffer *buf, ut64 loadaddr) {
-	ut64 user_baddr = bf->user_baddr;
-	ELFOBJ *res = Elf_(new_buf) (buf, user_baddr, bf->rbin->options.verbose);
+	ELFOBJ *res = Elf_(new_buf) (buf, bf->user_baddr, bf->rbin->options.verbose);
 	if (res) {
 	//	sdb_ns_set (sdb, "info", res->kv);
 		res->limit = bf->rbin->options.limit;
@@ -251,7 +250,7 @@ static RList* entries(RBinFile *bf) {
 		ptr->hpaddr = 0x18;  // e_entry offset in ELF header
 		ptr->hvaddr = UT64_MAX; // 0x18 + baddr (bf);
 
-		if (ptr->vaddr != (ut64)eo->ehdr.e_entry && Elf_(is_executable) (eo) && !eo->is_sbpf) {
+		if (ptr->vaddr != (ut64)eo->ehdr.e_entry && Elf_(is_executable) (eo)) {
 			R_LOG_ERROR ("Cannot determine entrypoint, using 0x%08" PFMT64x, ptr->vaddr);
 		}
 
@@ -454,6 +453,7 @@ static RBinReloc *reloc_convert(ELFOBJ* eo, RBinElfReloc *rel, ut64 got_addr) {
 	R_RETURN_VAL_IF_FAIL (eo && rel, NULL);
 	ut64 B = eo->baddr;
 	ut64 P = rel->rva; // rva has taken baddr into account
+
 	RBinReloc *r = R_NEW0 (RBinReloc);
 	r->import = NULL;
 	r->ntype = rel->type;
@@ -776,27 +776,23 @@ static RBinReloc *reloc_convert(ELFOBJ* eo, RBinElfReloc *rel, ut64 got_addr) {
 		ADD (32, 0);
 		break;
 	case EM_BPF:
-		if (!eo->is_sbpf) {
-			R_LOG_DEBUG ("Unimplemented BPF reloc type %d", rel->type);
-			break;
-		}
 	case EM_SBPF:
 		switch (rel->type) {
 		case R_BPF_64_64: // 64-bit immediate for lddw instruction
 			r->type = R_BIN_RELOC_64;
-			r->vaddr = B + rel->offset;
+			r->vaddr = P;
 			return r;
 		case R_BPF_64_RELATIVE: // PC relative 64-bit address
 			r->type = R_BIN_RELOC_64;
-			r->vaddr = B + rel->offset;
+			r->vaddr = P;
 			return r;
 		case R_BPF_64_32: // 32-bit function/syscall ID for call instruction
 			r->type = R_BIN_RELOC_32;
 			// The immediate value will be a function ID or syscall ID, not an address
-			r->vaddr = B + rel->offset;
+			r->vaddr = P;
 			return r;
 		default:
-			R_LOG_DEBUG ("Unimplemented sBPF reloc type %d", rel->type);
+			R_LOG_DEBUG ("Unimplemented BPF reloc type %d", rel->type);
 			break;
 		}
 		break;
@@ -822,7 +818,7 @@ static RBinReloc *reloc_convert(ELFOBJ* eo, RBinElfReloc *rel, ut64 got_addr) {
 // Helper macro for left bit rotation
 #define rotl32(x, n) (((x) << (n)) | ((x) >> (32 - (n))))
 
-// Firedancer-compatible Murmur3 32-bit hash function for sBPF syscalls
+// Murmur3 32-bit hash function for sBPF syscalls
 static ut32 murmur3_32(const char* data, ut32 len, ut32 seed) {
 	const ut32 c1 = 0xcc9e2d51U;
 	const ut32 c2 = 0x1b873593U;
@@ -1134,68 +1130,80 @@ static void _patch_reloc(ELFOBJ *bo, ut16 e_machine, RIOBind *iob, RBinElfReloc
 		break;
 	}
 	case EM_BPF: // CHECK: some older solana programs have set an ehdr.e_machine of EM_BPF
-		if (!bo->is_sbpf) {
-			R_LOG_DEBUG ("Unhandled BPF relocation type %d", rel->type);
+	case EM_SBPF: {
+		B = bo->user_baddr != UINT64_MAX ? bo->user_baddr : bo->baddr;
+		switch (rel->type) {
+		case R_BPF_64_64: { // 64-bit immediate for lddw instructions
+			// Read the current value from the immediate fields (addend)
+			// offset+4 = imm_lo, offset+8 = second instruction, offset+12 = imm_hi
+			ut32 vals[3];
+			ut64 addend = 0;
+			if (r_buf_fread_at (bo->b, rel->offset + 4, (ut8 *)vals, "iii", 1) == 12) {
+				addend = ((ut64)vals[2] << 32) | vals[0];
+			}
+			// V = symbol value + addend
+			V = S + addend;
+			// Normalize address: if V < base_addr, add base_addr
+			if (V < B) {
+				V += B;
+			}
+			// Write as split 32-bit values to immediate fields (rva+4 and rva+12)
+			r_write_le32 (buf, (ut32)(V & UT32_MAX));
+			iob->overlay_write_at (iob->io, rel->rva + 4 + B, buf, 4);
+			r_write_le32 (buf, (ut32)(V >> 32));
+			iob->overlay_write_at (iob->io, rel->rva + 12 + B, buf, 4);
 			break;
 		}
-		// fallthrough
-	case EM_SBPF: {
-		switch (rel->type) {
-		case R_BPF_64_64: // 64-bit immediate for lddw instructions
-			V = S + A;
-			// Add sBPF base address if result < base address
-			if (V < SBPF_PROGRAM_ADDR) {
-				V += SBPF_PROGRAM_ADDR;
-			}
-			// Write as split 32-bit values to immediate fields (offset+4 and offset+12)
-			r_write_le32 (buf, (ut32)(V & UT32_MAX));
-			iob->overlay_write_at (iob->io, rel->rva + 4, buf, 4);
-			r_write_le32 (buf, (ut32)(V >> 32));
-			iob->overlay_write_at (iob->io, rel->rva + 12, buf, 4);
-			break;
 
 		case R_BPF_64_RELATIVE: { // PC relative 64-bit address
-
-			// Check if relocation is in .text section
+			// Get addend - try explicit addend first, then implicit from file
+			ut64 addend = rel->addend;
+			// rel->offset is already a physical address (paddr), not a virtual address
+			ut64 paddr = rel->offset;
+			// Check if relocation is in .text section (check using paddr)
 			bool is_text = false;
 			ut64 text_start = Elf_(get_section_offset)(bo, ".text");
 			ut64 text_size = Elf_(get_section_size)(bo, ".text");
 			if (text_start != UT64_MAX && text_size != UT64_MAX) {
 				ut64 text_end = text_start + text_size;
-				is_text = (rel->offset >= text_start && rel->offset < text_end);
+				is_text = (paddr >= text_start && paddr < text_end);
 			}
+
 			if (is_text) {
 				// In .text: behave like R_BPF_64_64 but ignore symbol and handle addend
-				// Read implicit addend from both immediate fields (lddw instruction)
-				ut8 buf_lo[4], buf_hi[4];
-				iob->read_at (iob->io, rel->rva + 4, buf_lo, 4);
-				iob->read_at (iob->io, rel->rva + 12, buf_hi, 4);
-
-				ut32 va_lo = r_read_le32 (buf_lo);
-				ut32 va_hi = r_read_le32 (buf_hi);
-				ut64 va = ((ut64)va_hi << 32) | va_lo;
-
-				if (va != 0) {
-					// If looks like physical address, make it virtual
-					if (va < SBPF_PROGRAM_ADDR) {
-						va += SBPF_PROGRAM_ADDR;
+				// For lddw instruction, addend is split across two immediate fields
+				if (addend == 0 && paddr != UT64_MAX) {
+					// Read from the immediate fields - use physical offset
+					// paddr+4 = imm_lo, paddr+8 = second instruction, paddr+12 = imm_hi
+					ut32 vals[3];
+					if (r_buf_fread_at (bo->b, paddr + 4, (ut8 *)vals, "iii", 1) == 12) {
+						addend = ((ut64)vals[2] << 32) | vals[0];
 					}
-					// Write back to both immediate fields
-					r_write_le32 (buf_lo, (ut32)(va & 0xffffffff));
-					r_write_le32 (buf_hi, (ut32)(va >> 32));
-					iob->overlay_write_at (iob->io, rel->rva + 4, buf_lo, 4);
-					iob->overlay_write_at (iob->io, rel->rva + 12, buf_hi, 4);
 				}
+				// Apply relocation: B + addend
+				ut64 result = B + addend;
+				// Get the virtual address where this relocation should be written
+				ut64 write_vaddr = rel->rva + B;
+				// Write to both immediate fields (lddw instruction)
+				r_write_le32 (buf, (ut32)(result & UT32_MAX));
+				iob->overlay_write_at (iob->io, write_vaddr + 4, buf, 4);
+				r_write_le32 (buf, (ut32)(result >> 32));
+				iob->overlay_write_at (iob->io, write_vaddr + 12, buf, 4);
 			} else {
-				// Outside .text: do 64-bit write
-				ut8 buf_addend[4];
-				iob->read_at (iob->io, rel->rva + 4, buf_addend, 4);
-				ut32 va = r_read_le32 (buf_addend);
-				// Add base address
-				ut64 result = va + SBPF_PROGRAM_ADDR;
-				// Write back as 64-bit value
+				// Outside .text: Read 32-bit value, add base, write 64-bit result
+				// Read raw bytes and convert to little-endian 32-bit
+				ut8 raw_bytes[4];
+				r_buf_read_at (bo->b, paddr + 4, raw_bytes, 4);
+				ut32 refd_addr_32 = r_read_le32 (raw_bytes);
+				ut64 result = (ut64)refd_addr_32 + B;
+
+				R_LOG_DEBUG ("sBPF data reloc at 0x%"PFMT64x": read 0x%x from offset+4, result=0x%"PFMT64x,
+					rel->rva + B, refd_addr_32, result);
+
+				// Write full 64-bit value at rel->rva
+				ut64 write_vaddr = rel->rva + B;
 				r_write_le64 (buf, result);
-				iob->overlay_write_at (iob->io, rel->rva, buf, 8);
+				iob->overlay_write_at (iob->io, write_vaddr, buf, 8);
 			}
 			break;
 		}
@@ -1225,7 +1233,7 @@ static void _patch_reloc(ELFOBJ *bo, ut16 e_machine, RIOBind *iob, RBinElfReloc
 				ut32 hash_value = murmur3_32 (sym_name, strlen (sym_name), 0);
 				R_LOG_DEBUG ("sBPF R_BPF_64_32: syscall '%s' -> hash 0x%08x", sym_name, hash_value);
 				r_write_le32 (buf, hash_value);
-				iob->overlay_write_at (iob->io, rel->rva + 4, buf, 4);
+				iob->overlay_write_at (iob->io, rel->rva + 4 + B, buf, 4);
 			} else if (sym_addr > 0) {
 				// This is a regular function call - compute PC-relative offset
 				st64 current_pc = rel->rva / 8;  // Current instruction in units
@@ -1234,11 +1242,11 @@ static void _patch_reloc(ELFOBJ *bo, ut16 e_machine, RIOBind *iob, RBinElfReloc
 				R_LOG_DEBUG ("sBPF R_BPF_64_32: function '%s' at 0x%"PFMT64x" -> offset %d (0x%x)",
 					sym_name, sym_addr, offset, offset);
 				r_write_le32 (buf, offset);
-				iob->overlay_write_at (iob->io, rel->rva + 4, buf, 4);
+				iob->overlay_write_at (iob->io, rel->rva + 4 + B, buf, 4);
 			} else {
 				R_LOG_WARN ("sBPF R_BPF_64_32: symbol '%s' has no address", sym_name);
 				r_write_le32 (buf, 0);
-				iob->overlay_write_at (iob->io, rel->rva + 4, buf, 4);
+				iob->overlay_write_at (iob->io, rel->rva + 4 + B, buf, 4);
 			}
 			break;
 		}
@@ -1338,8 +1346,14 @@ static RList* patch_relocs(RBinFile *bf) {
 			}
 		}
 		// ut64 raddr = sym_addr? sym_addr: vaddr;
-		ut64 raddr = (sym_addr && sym_addr != UT64_MAX)? sym_addr: vaddr;
-		_patch_reloc (eo, eo->ehdr.e_machine, &b->iob, reloc, raddr, 0, plt_entry_addr);
+		// For sBPF, use rel->rva for relocations without symbols (like R_BPF_64_RELATIVE)
+		ut64 raddr;
+		if (sym_addr && sym_addr != UT64_MAX) {
+			raddr = sym_addr;
+		} else {
+			raddr = vaddr;
+		}
+		_patch_reloc (eo, eo->ehdr.e_machine, &b->iob, reloc, raddr, eo->baddr, plt_entry_addr);
 		ptr = reloc_convert (eo, reloc, n_vaddr);
 		if (!ptr) {
 			continue;
@@ -1348,9 +1362,12 @@ static RList* patch_relocs(RBinFile *bf) {
 		if (sym_addr && sym_addr != UT64_MAX) {
 			ptr->vaddr = sym_addr;
 		} else {
-			ptr->vaddr = vaddr;
-			ht_uu_insert (relocs_by_sym, reloc->sym, vaddr);
-			vaddr += cdsz;
+			// In sBPF, symbol relocations are handled independently as R_BPF_64_64
+			if (eo->ehdr.e_machine != EM_SBPF) {
+				ptr->vaddr = vaddr;
+				ht_uu_insert (relocs_by_sym, reloc->sym, vaddr);
+				vaddr += cdsz;
+			}
 		}
 		r_list_append (ret, ptr);
 	}
