@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2006-2024 - pancake */
+/* radare - LGPL - Copyright 2006-2025 - pancake */
 
 /* must be included first because of winsock2.h and windows.h */
 #include <r_socket.h>
@@ -171,9 +171,6 @@ static bool __listen_unix(RSocket *s, const char *file) {
 
 R_API RSocket *r_socket_new(bool is_ssl) {
 	RSocket *s = R_NEW0 (RSocket);
-	if (!s) {
-		return NULL;
-	}
 	s->is_ssl = is_ssl;
 	s->port = 0;
 #if __UNIX_
@@ -731,11 +728,15 @@ R_API bool r_socket_block_time(RSocket *s, bool block, int sec, int usec) {
 #elif R2__WINDOWS__
 	ioctlsocket (s->fd, FIONBIO, (u_long FAR*)&block);
 #endif
-	if (sec > 0 || usec > 0) {
-		struct timeval tv = {sec, usec};
-		if (setsockopt (s->fd, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv, sizeof (tv)) < 0) {
-			return false;
-		}
+	if (sec < 0) {
+		sec = 0;
+	}
+	if (usec < 0) {
+		usec = 0;
+	}
+	struct timeval tv = {sec, usec};
+	if (setsockopt (s->fd, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv, sizeof (tv)) < 0) {
+		return false;
 	}
 	return true;
 }
@@ -753,12 +754,18 @@ R_API int r_socket_flush(RSocket *s) {
 /* returns -1 on error, 0 is false, 1 is true */
 R_API int r_socket_ready(RSocket *s, int secs, int usecs) {
 	fd_set rfds;
-	struct timeval tv = {secs, usecs};
+	if (secs < 0) {
+		secs = 0;
+	}
+	if (usecs < 0) {
+		usecs = 0;
+	}
 	if (s->fd == R_INVALID_SOCKET) {
 		return -1;
 	}
 	FD_ZERO (&rfds);
 	FD_SET (s->fd, &rfds);
+	struct timeval tv = {secs, usecs};
 	return select (s->fd + 1, &rfds, NULL, NULL, &tv);
 }
 
